using System;
using System.Collections;
using NUnit.Framework;

namespace ServiceStack.Text.Tests
{
	[TestFixture]
	public class SpecialTypesTests
		: TestBase
	{
		[Test]
		public void Can_Serialize_Version()
		{
			Serialize(new Version());
			Serialize(Environment.Version);
		}

		public class JsonEntityWithPrivateGetter
		{
			public string Name { private get; set; }
		}

		public class JsonEntityWithNoProperties
		{
		}

		[Test]
		public void Can_Serialize_Type_with_no_public_getters()
		{
			Serialize(new JsonEntityWithPrivateGetter { Name = "Daniel" });
		}

		[Test]
		public void Can_Serialize_Type_with_no_public_properties()
		{
			Serialize(new JsonEntityWithNoProperties());
		}

		[Test]
		public void Can_Serialize_Type_with_ByteArray()
		{
			var test = new { Name = "Test", Data = new byte[] { 1, 2, 3, 4, 5 } };
			var json = JsonSerializer.SerializeToString(test);
			Assert.That(json, Is.EquivalentTo("{\"Name\":\"Test\",\"Data\":\"AQIDBAU=\"}"));
		}

		[Test]
		public void Can_Serialize_ByteArray()
		{
			var test = new byte[] { 1, 2, 3, 4, 5 };
			var json = JsonSerializer.SerializeToString(test);
			var fromJson = JsonSerializer.DeserializeFromString<byte[]>(json);

			Assert.That(test, Is.EquivalentTo(fromJson));
		}

	    [Test]
	    public void Can_Serialize_HashTable()
	    {
            var h = new Hashtable { { "A", 1 }, { "B", 2 } };
	        var fromJson = h.ToJson().FromJson<Hashtable>();
            Assert.That(fromJson.Count, Is.EqualTo(h.Count));
            Assert.That(fromJson["A"].ToString(), Is.EqualTo(h["A"].ToString()));
            Assert.That(fromJson["B"].ToString(), Is.EqualTo(h["B"].ToString()));
	    }

	    [Test]
	    public void Can_serialize_delegate()
	    {
            Action x = () => { };

            Assert.That(x.ToJson(), Is.Null);
            Assert.That(x.ToJsv(), Is.Null);
            Assert.That(x.Dump(), Is.Not.Null);
	    }

        string MethodWithArgs(int id, string name)
        {
            return null;
        }
        
	    [Test]
	    public void Does_dump_delegate_info()
	    {
            Action d = Can_Serialize_ByteArray;
            Assert.That(d.Dump(), Is.EqualTo("Void Can_Serialize_ByteArray()"));

	        Func<int, string, string> methodWithArgs = MethodWithArgs;
            Assert.That(methodWithArgs.Dump(), Is.EqualTo("String MethodWithArgs(Int32 arg1, String arg2)"));

            Action x = () => { };
            Assert.That(x.Dump(), Is.EqualTo("Void <Does_dump_delegate_info>b__4()"));
        }
	}
}